#include <stdlib.h>
#include <stdio.h>

#include "wilson.h"

#define Ncomplex 2
#define Ncolour 3
#define Nspin2 2
#define Nspin4 4
#define Ncb 2
#define ND 4
#define Re 0
#define Im 1

#define   MYRAND_MAX        (2147483647)
static unsigned long long rand_seed;
void mysrand(unsigned int seed)
{
  rand_seed = seed;
}
 
int myrand(void)
{
  int r;
 
  rand_seed = rand_seed *  6364136223846793005LL + 1;
  return (int) ( ( rand_seed  >> 32 ) & MYRAND_MAX );
}

void print_result(char *name,Float *ans, int nword)
{
  int i;
  FILE *result;
  result = fopen(name,"w");
  for (i=0;i<nword;i++) {
    fprintf(result,"%16.8le\n",ans[i]);
  }
  fclose(result);
}

struct Spinor { 
  Float vals[ND][Ncolour][Ncomplex]; 
};
typedef struct Spinor Spin4;


typedef struct { Float vals[Nspin4][Ncolour][Ncolour][Ncomplex];} Gauge;

void RanFloat(Float *f,int n)
{
  for ( int i=0;i<n;i++ ) {
    f[i] = (Float)myrand()/MYRAND_MAX;
  }
}


int main (int argc,char **argv )
{
  int lx,ly,lz,lt;
  int lxyzt;
  int sx;

  if ( argc != 5 ) { 
    printf("Usage: %s lx ly lz lt\n All must be even",argv[0]);
    exit(-1);
  }

  lx = atoi(argv[1]);
  ly = atoi(argv[2]);
  lz = atoi(argv[3]);
  lt = atoi(argv[4]);

  if( (lx&0x1)||(ly&0x1)||(lz&0x1)||(lt&0x1) ) { 
    printf("Usage: %s lx ly lz lt\nAll must be even\n",argv[0]);
    printf("%d %d %d %d\n",lx,ly,lz,lt);
    fflush(stdout);
    exit(-1);
  }

  sx = lx/2;
  lxyzt = sx*ly*lz*lt;

  Spin4 *Chi = new Spin4[lxyzt*Ncb];
  Spin4 *PSI = new Spin4[lxyzt*Ncb];
  Gauge *U = new Gauge[lxyzt*Ncb];

  srand(1);
  RanFloat((Float *)Chi,lxyzt*24*Ncb);
  RanFloat((Float *)PSI,lxyzt*24*Ncb);
  RanFloat((Float *)U,lxyzt*18*4*Ncb);

  Wilson wil;

  wil.ptr = (int *) malloc(4*sizeof(int));
  wil.ptr[0] = lx;
  wil.ptr[1] = ly;
  wil.ptr[2] = lz;
  wil.ptr[3] = lt;
  /*
  wil.local_latt[0]  = lx;
  wil.local_latt[1]  = ly;
  wil.local_latt[2]  = lz;
  wil.local_latt[3]  = lt;
  wil.local_comm[0]  = 0;
  wil.local_comm[1]  = 0;
  wil.local_comm[2]  = 0;
  wil.local_comm[3]  = 0;
  */
  wilson_init(&wil);

  wilson_dslash((Float *)Chi, 
	       (Float *)U, 
	       (Float *)PSI, 
	       0,
	       0,&wil);

  print_result("Chi_cb0",(Float *)Chi,24*lxyzt);
  wilson_dslash((Float *)Chi, 
		(Float *)U, 
		(Float *)PSI, 
		0,
		1,&wil);
  print_result("Chi_cb0_dag",(Float *)Chi,24*lxyzt);
  wilson_dslash((Float *)Chi, 
		(Float *)U, 
		(Float *)PSI, 
		1,
		0,&wil);
  print_result("Chi_cb1",(Float *)Chi,24*lxyzt);
  wilson_dslash((Float *)Chi, 
		(Float *)U, 
		(Float *)PSI, 
		1,
		1,&wil);
  print_result("Chi_cb1_dag",(Float *)Chi,24*lxyzt);

  exit(0);
}







